
#include <cassert>   /* assert */
#include <algorithm> /* std::min and std::max */

#include "account.h"

Account::Account(unsigned int number, double balance): 
    m_number(number), m_balance(balance) {
}
void Account::credit(double amount) {
    assert(amount>=0.0);
    m_balance += amount;
}
double Account::debit(double amount) {
    assert(amount>=0.0);
    m_balance -= amount;
    return amount;
}
unsigned int Account::operator()() const {
    return m_number;
}

CurrentAccount::CurrentAccount(unsigned int number, double balance):
    Account(number, balance) {
}
double CurrentAccount::debit(double amount) {
    amount = std::max(std::min(amount, m_balance), 0.0);
    return Account::debit(amount);
}
void CurrentAccount::print(std::ostream& os) const {
    os << "CurrentAccount[number=" << this->operator()() 
       << ", balance=" << m_balance << "€]";
}

SavingAccount::SavingAccount(unsigned int number, double balance, double interest_rate):
    Account(number, balance), m_interest_rate(interest_rate) {
}
void SavingAccount::print(std::ostream& os) const {
    os << "SavingAccount[number=" << (*this)() 
       << ", balance=" << m_balance << "€"
       << ", rate=" << m_interest_rate << "%]";
}

double SavingAccount::add_interests() {
    double interests = m_balance * m_interest_rate / 100.0;
    this->credit(interests);
    return interests;
}

BlockedSavingAccount::BlockedSavingAccount(unsigned int number, double balance):
    SavingAccount(number, balance, IRATEBLOCKED) {
}
double BlockedSavingAccount::debit(double amount) {
    return -1.0;
}

UnblockedSavingAccount::UnblockedSavingAccount(unsigned int number, double balance):
    SavingAccount(number, balance, IRATEUNBLOCKED) {
}
double UnblockedSavingAccount::debit(double amount) {
    amount = (amount > m_balance ? 0.0 : amount);
    return Account::debit(amount);
}

std::ostream& operator<<(std::ostream& os, const Account& account) { 
    os << "  "; account.print(os); os<<std::endl; return os; 
}
