
#include <cassert>  /* assert */
#include "client.h"
        
Client::Client(std::string name, unsigned int identifier, unsigned int max_accounts):
    m_name(name), m_identifier(identifier), 
    m_ncurrent(0), m_nmax(max_accounts), 
    m_accounts(nullptr) {
    m_accounts = new Account*[max_accounts];
    for (unsigned int i=0; i<max_accounts; i++) {
        m_accounts[i] = nullptr;
    }
}

Client::Client(const Client& other):
    m_name(other.m_name), m_identifier(other.m_identifier),
    m_ncurrent(other.m_ncurrent), m_nmax(other.m_nmax),
    m_accounts(nullptr) {

    m_accounts = new Account*[other.m_nmax];
    for (unsigned int i=0; i<other.m_nmax; i++) {
        m_accounts[i] = (i<other.m_ncurrent ? other.m_accounts[i]->clone() : nullptr);
    }
}

Client::~Client() {
    for (unsigned int i=0; i<m_ncurrent; i++) {
        delete m_accounts[i];
    }
    delete[] m_accounts;
}

unsigned int Client::identifier() const {
    return m_identifier;
}

void Client::create_account(unsigned int type) {
    if (type==0) {
        /* prompt the user for an account */
        std::cout << "Creation of a "
                  << "current account(1), unblocked savings account(2) "
                  << "or blocked savings account(3) ? \n";
        std::cin >> type;
    }

    Account* account = nullptr;
    const unsigned int number = m_ncurrent;
    const unsigned int balance = 0u;

    switch(type) {
        case 1:
            account = new CurrentAccount(number, balance);
            break;
        case 2:
            account = new BlockedSavingAccount(number, balance);
            break;
        case 3:
            account = new UnblockedSavingAccount(number, balance);
            break;
        default:
            std::cerr << "Unsupported account type " << type << '.' << std::endl;
    }

    if (account != nullptr) {
        assert(m_accounts[m_ncurrent] == nullptr);
        m_accounts[m_ncurrent] = account;
        m_ncurrent += 1;
    }
}

double Client::debit_account(unsigned int k, double amount) {
    assert(k < m_ncurrent);
    assert(m_accounts[k] != nullptr);
    return m_accounts[k]->debit(amount);
}

void Client::credit_account(unsigned int k, double amount) {
    assert(k < m_ncurrent);
    assert(m_accounts[k] != nullptr);
    return m_accounts[k]->credit(amount);
}

std::ostream &operator<<(std::ostream &os, const Client& client) {
    os << " [Client]" << std::endl;
    os << "   name:       " << client.m_name       << std::endl;
    os << "   identifier: " << client.m_identifier << std::endl;
    if (client.m_ncurrent>0) {
        for (unsigned int i=0; i<client.m_ncurrent; i++) {
            os << "   " << *client.m_accounts[i];
        }
    }
    else {
        os << "  -- no account opened" << std::endl;
    }
    return os;
}
